import os
import requests
from loguru import logger

class BingSearchEngine:
    BING_MKT = "en-US"
    BING_SEARCH_V7_ENDPOINT = "https://api.bing.microsoft.com/v7.0/search"
    DEFAULT_SEARCH_ENGINE_TIMEOUT = 5
    REFERENCE_COUNT = 8

    def __init__(self):
        self.subscription_key = os.getenv('BING_SUBSCRIPTION_KEY')

    def search(self, query: str):
        params = {"q": query, "mkt": self.BING_MKT}
        response = requests.get(
            self.BING_SEARCH_V7_ENDPOINT,
            headers={"Ocp-Apim-Subscription-Key": self.subscription_key},
            params=params,
            timeout=self.DEFAULT_SEARCH_ENGINE_TIMEOUT,
        )
        if not response.ok:
            logger.error(f"{response.status_code} {response.text}")
        json_content = response.json()
        try:
            contexts = json_content["webPages"]["value"][:self.REFERENCE_COUNT]
        except KeyError:
            logger.error(f"Error encountered: {json_content}")
            return []
        return contexts
    


import os
import openai
from bs4 import BeautifulSoup
from askyou.utils import requests_retry_session

class Summarizer:
    def __init__(self):
        self.client = openai.OpenAI(
            base_url=os.getenv('OPENAI_BASE_URL'),
            api_key=os.getenv('OPENAI_API_KEY'),
        )

    def fetch_and_summarize(self, url):
        content = self._fetch_content(url)
        return self._summarize(content)

    def _fetch_content(self, url):
        response = requests_retry_session().get(url, timeout=10)
        soup = BeautifulSoup(response.text, 'html.parser')
        
        for meta in soup.find_all('meta'):
            if 'content' in meta.attrs:
                content = meta['content']
                charset_pos = content.lower().find('charset=')
                if charset_pos != -1:
                    charset = content[charset_pos + 8:].split(';')[0]
                    response.encoding = 'gbk' if charset == 'GB2312' else 'utf-8'
                    break

        html = response.text
        soup = BeautifulSoup(html, 'html.parser')
        
        text_content_list = [url]
        for tag in soup.find_all(['p', 'code']):
            text_content_list.append(tag.get_text())
        
        return ' '.join(text_content_list)

    def _summarize(self, content):
        response = self.client.chat.completions.create(
            model=os.getenv('MODEL'),
            messages=[
                {"role": "system", "content": '你是总结文章的专家，我会给你一篇文章，你需要将文章进行总结，请尽量详细易懂，使用中文。'},
                {"role": "user", "content": content},
            ],
            max_tokens=4056,
            stream=True,
            temperature=0.6,
        )
        
        summary = ''
        for chunk in response:
            if chunk.choices[0].delta.content is not None:
                summary += chunk.choices[0].delta.content
        return summary
    


import os
import openai

class Translator:
    def __init__(self):
        self.client = openai.OpenAI(
            base_url=os.getenv('OPENAI_BASE_URL'),
            api_key=os.getenv('OPENAI_API_KEY'),
        )

    def translate(self, text, lang='cn'):
        lang_prompt = '请把以下句子翻译成中文:' if lang == 'cn' else '日本語に翻訳してください:'
        
        translation = self.client.chat.completions.create(
            model=os.getenv('MODEL'),
            messages=[
                {"role": "system", "content": 'You are an expert in translation and I will give you a sentence that you will translate into another language. You just need to return the translation without any other extra words.'},
                {"role": "user", "content": lang_prompt + text},
            ],
            stream=True,
            temperature=0.5,
        )
        
        result = ''
        for chunk in translation:
            if chunk.choices[0].delta.content is not None:
                result += chunk.choices[0].delta.content
        return result

    def optimize_query(self, query, lang='cn'):
        if lang != 'cn':
            query = self.translate(query, lang)
        
        better_query = self.client.chat.completions.create(
            model='Qwen/Qwen2-72B-Instruct',
            messages=[
                {"role": "system", "content": 'You are an expert in search engine utilization and I will give you a question that you will optimize into a search term more suitable for the search engines to hand over to me. Usually it will be in the form of a keyword with a space so that you can cover most of the answers. You just need to return the optimization results without any other extra words.'},
                {"role": "user", "content": query},
            ],
            stream=True,
            temperature=0.3,
        )
        
        result = ''
        for chunk in better_query:
            if chunk.choices[0].delta.content is not None:
                result += chunk.choices[0].delta.content
        return result.strip('"')
    


import os
from dotenv import load_dotenv

from askyou.cli import cli, check_tokens

def main():
    project_root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

    # 构建 .env 文件的路径
    env_path = os.path.join(project_root, '.env')

    # 加载 .env 文件
    load_dotenv(env_path)
    check_tokens()
    cli()

if __name__ == "__main__":
    main()


import click
import re
from askyou.search_engine import BingSearchEngine
from askyou.translator import Translator
from askyou.summarizer import Summarizer
from askyou.config_manager import ConfigManager
import concurrent.futures
import time

import openai
import os

@click.group(invoke_without_command=True)
@click.pass_context
def cli(ctx):
    if ctx.invoked_subcommand is None:
        ctx.invoke(query_function)

@cli.command(name='set-token')
@click.argument('key')
@click.argument('value')
def set_token(key, value):
    """This command is used to set an API token and model."""
    config_manager = ConfigManager()
    """Set an API token."""
    config_manager.set_token(key, value)
    click.echo(f"Token {key} has been set.")

def check_tokens():
    config_manager = ConfigManager()
    missing_tokens = config_manager.check_tokens()
    if missing_tokens:
        click.echo("The following tokens are missing:")
        for token in missing_tokens:
            click.echo(f"- {token}")
        click.echo("Please set them using the 'set-token' command.")
        return False
    return True

@cli.command(name='about')
@click.option('--lang', default='cn', help='language of the search query')
@click.option('--all', is_flag=True, help='fetch the all context')
@click.argument('query', default='Who said "live long and prosper"?')
def query_function(query, lang, all):
    """This command is used to query the AI search engine."""
    query = query or "Who said 'live long and prosper'?"
    query = re.sub(r"\[/?INST\]", "", query)
    
    translator = Translator()
    search_query = translator.optimize_query(query, lang)
    print('search_query:', search_query)
    
    search_engine = BingSearchEngine()
    contexts = search_engine.search(search_query)
    
    if all:
        start_time = time.time()
        summarizer = Summarizer()
        
        def process_context(context):
            url = context['url']
            context['snippet'] += summarizer.fetch_and_summarize(url)

        with concurrent.futures.ThreadPoolExecutor() as executor:
            futures = [executor.submit(process_context, context) for context in contexts]
            concurrent.futures.wait(futures)
        
        end_time = time.time()
        execution_time = end_time - start_time
        print(f"Execution time: {execution_time} seconds")

    system_prompt = _create_system_prompt(contexts)
    
    client = openai.OpenAI(
        base_url=os.getenv('OPENAI_BASE_URL'),
        api_key=os.getenv('OPENAI_API_KEY'),
    )
    
    response = client.chat.completions.create(
        model=os.getenv('MODEL'),
        messages=[
            {"role": "system", "content": system_prompt},
            {"role": "user", "content": query},
        ],
        max_tokens=1024,
        stream=True,
        temperature=0.8,
    )

    for chunk in response:
        if chunk.choices[0].delta.content is not None:
            print(chunk.choices[0].delta.content, end='')
    print('\n')
    print('\n'.join([f"[[citation:{i+1}]] {c['url']}" for i, c in enumerate(contexts)]))

def _create_system_prompt(contexts):
    config_manager = ConfigManager()
    config = config_manager.load_config()
    rag_query_text = config['rag_query_text']
    return rag_query_text.format(
        context="\n\n".join(
            [f"[[citation:{i+1}]] {c['snippet']}" for i, c in enumerate(contexts)]
        )
    )

@cli.command(name='whoami')
@click.argument('prompt')
def whoami(prompt):
    """This command is used to update the prompt."""
    config_manager = ConfigManager()
    config_manager.update_config(prompt)

@cli.command(name='url')
@click.argument('url')
def summarize_url(url):
    """This command is used to process the given URL."""
    summarizer = Summarizer()
    summary = summarizer.fetch_and_summarize(url)
    print(summary)

if __name__ == "__main__":
    cli()



import os
import json
from dotenv import load_dotenv, set_key

class ConfigManager:
    def __init__(self):
        self.env_file = '.env'
        load_dotenv(self.env_file)

    def get_token(self, key):
        return os.getenv(key)

    def set_token(self, key, value):
        set_key(self.env_file, key, value)
        os.environ[key] = value

    def check_tokens(self):
        required_tokens = ['BING_SUBSCRIPTION_KEY', 'OPENAI_API_KEY', 'OPENAI_BASE_URL', 'MODEL']
        missing_tokens = [token for token in required_tokens if not self.get_token(token)]
        return missing_tokens
    
    def load_config(self):
        project_root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
        config_path = os.path.join(project_root, 'askyou/config.json')
        with open(config_path, 'r', encoding='utf-8') as f:
            config = json.load(f)
        return config
    
    def update_config(self, new_text):
        project_root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
        config_path = os.path.join(project_root, 'askyou/config.json')
        with open(config_path, 'r', encoding='utf-8') as f:
            config = json.load(f)
        target_sentence = 'You will be given a set of related contexts to the question'
        index = config['rag_query_text'].find(target_sentence)
        if index != -1:
            config['rag_query_text'] = new_text + config['rag_query_text'][index:]
        with open(config_path, 'w', encoding='utf-8') as f:
            json.dump(config, f, ensure_ascii=False, indent=4)


from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry
from requests import Session

def requests_retry_session(
    retries=3,
    backoff_factor=0.3,
    status_forcelist=(500, 502, 504),
    session=None,
):
    session = session or Session()
    retry = Retry(
        total=retries,
        read=retries,
        connect=retries,
        backoff_factor=backoff_factor,
        status_forcelist=status_forcelist,
    )
    adapter = HTTPAdapter(max_retries=retry)
    session.mount('http://', adapter)
    session.mount('https://', adapter)
    return session


from setuptools import setup, find_packages

setup(
    name="askyou",
    version="0.1.0",
    author="xie weicong",
    author_email="xieweicong95@gmail.com",
    description="A ai search cli",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/xieweicong/askyou",
    packages=find_packages(),
    package_data={
        'askyou': ['*.json'],
    },
    install_requires=[
        "click",
        "requests",
        "beautifulsoup4",
        "openai",
        "python-dotenv",
        "loguru",
    ],  # 项目依赖
    entry_points={
        "console_scripts": [
            "askyou=askyou.main:main",  # 这定义了命令行入口点
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",  # 指定 Python 版本要求
)